"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildNameValueObjectArray = exports.subJoinByRelFunc = exports.subJoin = exports.innerJoin = exports.leftJoin = exports.arrangeObjArray = void 0;
const lodash_differencewith_1 = __importDefault(require("lodash.differencewith"));
function arrangeObjArray(from, to, compare = defaultCompare) {
    const remove = (0, lodash_differencewith_1.default)(from, to, compare);
    const add = (0, lodash_differencewith_1.default)(to, from, compare);
    return {
        remove,
        add
    };
}
exports.arrangeObjArray = arrangeObjArray;
function defaultCompare(oldObj, newObj) {
    return oldObj.id == newObj.id;
}
function leftJoin(rows1, rows2, rels, fillEmptyField = true, rows2Fields) {
    let results = [];
    let buildRow2Fields = false;
    if (!rows2Fields) {
        buildRow2Fields = true;
        rows2Fields = [];
    }
    for (let idx1 in rows1) {
        let row1 = rows1[idx1];
        let noHit = true;
        for (let idx2 in rows2) {
            let hit = true;
            let row2 = rows2[idx2];
            if (buildRow2Fields) {
                Object.keys(row2).forEach((colKey) => {
                    if (!(rows2Fields === null || rows2Fields === void 0 ? void 0 : rows2Fields.includes(colKey))) {
                        rows2Fields === null || rows2Fields === void 0 ? void 0 : rows2Fields.push(colKey);
                    }
                });
            }
            for (let relIdx in rels) {
                const rel = rels[relIdx];
                if (row1[rel.leftField] != row2[rel.rightField]) {
                    hit = false;
                    break;
                }
            }
            if (hit) {
                noHit = false;
                const joinRow = Object.assign({}, row1);
                Object.keys(row2).forEach((key) => {
                    if (!joinRow[key]) {
                        joinRow[key] = row2[key];
                    }
                });
                results.push(joinRow);
            }
        }
        if (noHit) {
            const joinRow = Object.assign({}, row1);
            if (fillEmptyField) {
                rows2Fields.forEach((field) => {
                    if (!joinRow[field]) {
                        joinRow[field] = null;
                    }
                });
            }
            results.push(joinRow);
        }
    }
    return results;
}
exports.leftJoin = leftJoin;
function innerJoin(rows1, rows2, rels) {
    let results = [];
    for (let idx1 in rows1) {
        let row1 = rows1[idx1];
        for (let idx2 in rows2) {
            let row2 = rows2[idx2];
            let hit = true;
            for (let relIdx in rels) {
                const rel = rels[relIdx];
                if (row1[rel.leftField] != row2[rel.rightField]) {
                    hit = false;
                    break;
                }
            }
            if (hit) {
                const joinRow = Object.assign({}, row1);
                Object.keys(rows2[0]).forEach((key) => {
                    if (!joinRow[key]) {
                        joinRow[key] = row2[key];
                    }
                });
                results.push(joinRow);
            }
        }
    }
    return results;
}
exports.innerJoin = innerJoin;
function subJoin(mainRows, subRows, rels, subItemsKey = 'children') {
    for (let mainIdx in mainRows) {
        let mainRow = mainRows[mainIdx];
        let children = mainRow.children || [];
        for (let subIdx in subRows) {
            let subRow = subRows[subIdx];
            let hit = true;
            for (let relIdx in rels) {
                const rel = rels[relIdx];
                if (mainRow[rel.leftField] != subRow[rel.rightField]) {
                    hit = false;
                    break;
                }
            }
            if (hit) {
                children.push(subRow);
                subRow._xrr = `${Number(mainIdx) + 1}.${children.length}`;
            }
        }
        mainRow._xrr = Number(mainIdx) + 1;
        mainRow[subItemsKey] = children;
    }
    return mainRows;
}
exports.subJoin = subJoin;
function subJoinByRelFunc(mainRows, subRows, relFunc, subItemsKey = 'children', nullSafety = false) {
    for (let mainIdx in mainRows) {
        let mainRow = mainRows[mainIdx];
        let children = [];
        for (let subIdx in subRows) {
            let subRow = subRows[subIdx];
            if (relFunc(mainRow, subRow)) {
                children.push(subRow);
            }
        }
        if (nullSafety || (children && children.length > 0)) {
            mainRow[subItemsKey] = children;
        }
    }
    return mainRows;
}
exports.subJoinByRelFunc = subJoinByRelFunc;
function buildNameValueObjectArray(data) {
    if (!data) {
        return [];
    }
    let nvObjArr = [];
    let orgObj = {};
    if (Array.isArray(data)) {
        if (data.length == 0) {
            return [];
        }
        orgObj = data[0];
    }
    else {
        orgObj = data;
    }
    Object.keys(orgObj).forEach((key) => {
        const nvObj = {};
        nvObj['name'] = key;
        nvObj['value'] = orgObj[key];
        nvObjArr.push(nvObj);
    });
    return nvObjArr;
}
exports.buildNameValueObjectArray = buildNameValueObjectArray;
