"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dataColFlatExtend = exports.dataColExtend = void 0;
const lodash_sortby_1 = __importDefault(require("lodash.sortby"));
const object_1 = require("./object");
function dataColExtend(rows = [], groupByColProps = [], type, colHeaderProp, matchDynHeaderProps = []) {
    if (!type) {
        throw new Error('ERROR_type_UNDEFINED');
    }
    if (!colHeaderProp) {
        throw new Error('ERROR_matchDynHeaderProps_UNDEFINED');
    }
    rows = (0, lodash_sortby_1.default)(rows, groupByColProps);
    matchDynHeaderProps = (0, lodash_sortby_1.default)(matchDynHeaderProps);
    const newRows = [];
    let newRow = {};
    rows.forEach((r) => {
        const groupBySame = (0, object_1.compareObjWithKeys)(r, newRow, groupByColProps);
        if (groupBySame) {
            if (type.toUpperCase() === 'L1') {
                l1Extend(matchDynHeaderProps, colHeaderProp, newRow, r);
            }
            else if (type.toUpperCase() === 'L2') {
                l2Extend(matchDynHeaderProps, colHeaderProp, newRow, r);
            }
        }
        else {
            newRow = new Object();
            const keys = Object.keys(r);
            keys.forEach((k) => {
                if (groupByColProps.includes(k)) {
                    newRow[k] = r[k];
                }
            });
            if (type.toUpperCase() === 'L1') {
                l1Extend(matchDynHeaderProps, colHeaderProp, newRow, r);
            }
            else if (type.toUpperCase() === 'L2') {
                l2Extend(matchDynHeaderProps, colHeaderProp, newRow, r);
            }
            newRows.push(newRow);
        }
    });
    return newRows;
}
exports.dataColExtend = dataColExtend;
function dataColFlatExtend(rows = [], groupByColProps = [], matchDynHeaderProps = []) {
    rows = (0, lodash_sortby_1.default)(rows, groupByColProps);
    const newRows = [];
    let newRow = {};
    rows.forEach((r, idx) => {
        const groupBySame = (0, object_1.compareObjWithKeys)(r, newRow, groupByColProps);
        if (groupBySame) {
            flatExtend(matchDynHeaderProps, newRow, r, idx + 1);
        }
        else {
            newRow = new Object();
            const keys = Object.keys(r);
            keys.forEach((k) => {
                if (groupByColProps.includes(k)) {
                    newRow[k] = r[k];
                }
            });
            flatExtend(matchDynHeaderProps, newRow, r, idx + 1);
            newRows.push(newRow);
        }
    });
    return newRows;
}
exports.dataColFlatExtend = dataColFlatExtend;
function flatExtend(matchDynHeaderProps, newRow, r, rownum) {
    matchDynHeaderProps.forEach((prop) => {
        const i = rownum % matchDynHeaderProps.length;
        const seq = i == 0 ? matchDynHeaderProps.length : i;
        newRow[`${prop}_${seq}`] = r[prop];
    });
}
function l1Extend(matchDynHeaderProps, colHeaderProp, newRow, r) {
    matchDynHeaderProps.forEach((prop) => {
        newRow[`${r[colHeaderProp]}_${prop}`] = r[prop];
    });
}
function l2Extend(matchDynHeaderProps, colHeaderProp, newRow, r) {
    matchDynHeaderProps.forEach((prop) => {
        newRow[`${prop}_${r[colHeaderProp]}`] = r[prop];
    });
}
