"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.systemPath = exports.createFile = exports.softDeleteFile = exports.deleteFile = exports.viewDirectoryContents = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
function viewDirectoryContents(directoryPath, fileFullName = false) {
    directoryPath = buildFileFullPath(directoryPath);
    fs_1.default.accessSync(directoryPath, fs_1.default.constants.R_OK);
    const fileNames = fs_1.default.readdirSync(directoryPath);
    let result = [];
    fileNames.forEach((fileName) => {
        if (fileFullName) {
            fileName = buildFileFullPath(directoryPath, fileName);
        }
        result.push({ fileName });
    });
    return { items: result, total: fileNames.length };
}
exports.viewDirectoryContents = viewDirectoryContents;
function deleteFile(filePath, fileName) {
    const fileFullName = buildFileFullPath(filePath, fileName);
    fs_1.default.accessSync(fileFullName, fs_1.default.constants.W_OK);
    fs_1.default.unlinkSync(fileFullName);
}
exports.deleteFile = deleteFile;
function softDeleteFile(filePath, fileName) {
    let oldFileFullName = `${filePath}/${fileName}`;
    let newFileFullName = `${filePath}/deleted_${fileName}`;
    if (process.platform == 'win32') {
        oldFileFullName = oldFileFullName.split(path_1.default.posix.sep).join(path_1.default.sep);
    }
    fs_1.default.accessSync(oldFileFullName, fs_1.default.constants.W_OK);
    fs_1.default.renameSync(oldFileFullName, newFileFullName);
}
exports.softDeleteFile = softDeleteFile;
function createFile(filePath, fileName, fileData) {
    filePath = buildFileFullPath(filePath);
    if (!fs_1.default.existsSync(filePath)) {
        fs_1.default.mkdirSync(filePath, { recursive: true });
    }
    const fileFullName = buildFileFullPath(filePath, fileName);
    fs_1.default.writeFileSync(fileFullName, fileData);
}
exports.createFile = createFile;
function systemPath(filePath, sysType) {
    const platform = sysType || process.platform;
    let systemPath = path_1.default.resolve(filePath);
    if (platform == 'win32') {
        systemPath = systemPath.split(path_1.default.posix.sep).join(path_1.default.win32.sep);
    }
    else {
        systemPath = systemPath.split(path_1.default.win32.sep).join(path_1.default.posix.sep);
    }
    return systemPath;
}
exports.systemPath = systemPath;
function buildFileFullPath(filePath, fileName) {
    let fileFullPath = path_1.default.resolve(filePath, fileName || '');
    if (process.platform == 'win32') {
        fileFullPath = fileFullPath.split(path_1.default.posix.sep).join(path_1.default.sep);
    }
    return fileFullPath;
}
